/*-
 * Copyright (c) 2006 Stanislav Sedov <stas@FreeBSD.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $MBSDlabs: devcpu/cpu_microcode_tool/cpu_microcode_tool.h,v 1.3 2006/12/08 22:34:03 stas Exp $
 *
 */

#ifndef _CPU_MICROCODE_TOOL_H_
#define _CPU_MICROCODE_TOOL_H_

/* #define DEBUG */
#ifndef DEBUG
# define ASSERT(x)
#else /* DEBUG */
# define ASSERT(x) assert(x)
#endif

#ifdef DEBUG
# define WARNX(level, ...)						\
	if ((level) <= verbose_level) {					\
		fprintf(stderr, "%s:%d ", __FILE__, __LINE__);		\
		warnx(__VA_ARGS__);					\
	}
# define WARN(level, ...)						\
	if ((level) <= verbose_level) {					\
		fprintf(stderr, "%s:%d ", __FILE__, __LINE__);		\
		warn(__VA_ARGS__);					\
	}
#else
# define WARNX(level, ...)						\
	if ((level) <= verbose_level) {					\
		warnx(__VA_ARGS__);					\
	}
# define WARN(level, ...)						\
	if ((level) <= verbose_level) {					\
		warn(__VA_ARGS__);					\
	}
#endif

/* Macros for freeing SLISTs, probably must be in /sys/queue.h */
#define SLIST_FREE(head, field, freef) do {		\
    typeof(SLIST_FIRST(head)) __elm0;			\
    typeof(SLIST_FIRST(head)) __elm;			\
    SLIST_FOREACH_SAFE(__elm, (head), field, __elm0)	\
        (void)(freef)(__elm);				\
} while(0);

typedef int mu_add_t(int unit);
typedef void mu_free_t (void);
typedef void mu_update_t(const char *image);

extern int verbose_level;

#endif  /* _CPU_MICROCODE_TOOL_H_ */
