#!/usr/bin/env ruby
#
# $MBSDlabs: devcpu/tools/extract_mcodes.rb,v 1.2 2007/02/01 19:20:59 stas Exp $
#

require 'tempfile'
require 'lhalib'
require 'mmap'

class Splitter
    def initialize(file, pattern)
        @file = file
        @pattern = pattern
        self
    end
    def extract(dir = Dir.pwd)
	mmap = Mmap.new(@file, "r")

	first = mmap.index(@pattern)
	return nil if first.nil?

	while last = mmap.index(@pattern, first + 1)
	    ofile = File.new(dir + "/part#{$part}", \
			     File::CREAT|File::TRUNC|File::RDWR, 0644)

	    
	    ofile.write(mmap[first ... last])
	    ofile.close

	    yield ofile.path if block_given?
	    first = last
	    $part += 1
	end

	    last = mmap.length
	    ofile = File.new(dir + "/part#{$part}", \
			     File::CREAT|File::TRUNC|File::RDWR, 0644)
	    
	    ofile.write(mmap[first ... last])
	    ofile.close

	    yield ofile.path if block_given?
	    $part += 1
    end
end

class AwardBiosReader < Splitter
    def initialize(file)
	super(file, /.{2,2}-lh[[:digit:]]-/)
        self
    end
end

class AmdCpucodeReader < Splitter
    def initialize(file)
	pattern = /.{29,29}#{[0xaa, 0xaa, 0xaa].pack("ccc")}/
	super(file, pattern)
	self
    end
end

class IntelCpucodeReader
    def initialize(file)
	@file = file
	@hlen = 48
	self
    end

    def make_checksum(data)
	unpacked = data.unpack("I#{data.length/4}")
	sum = 0

	unpacked.each { |word| sum += word }
	sum
    end

    def extract_single(file, mmap, offset)
        hversion, revision, date, sign, cksum, hrev, cflags, dsize, tsize = \
    	mmap[offset ... @hlen + offset].unpack("IiIIIIIII")

        return nil if tsize.nil? or hversion != 1 or hrev !=1

	dsize = 2000 if dsize.zero?	# according to spec

    	tsize = dsize + @hlen if tsize.zero?
	return nil if (tsize + offset > mmap.length) or (tsize % 4 != 0)

	# Check contol sum
	checksum = make_checksum(mmap[offset ... offset + tsize])
	return nil if checksum == 2**32

	ofile = File.new(file, File::CREAT|File::TRUNC|File::RDWR, 0644)
	ofile.write(mmap[offset ... offset + tsize])
	ofile.close

	return offset + tsize
    end

    def extract(dir = Dir.pwd)
	mmap = Mmap.new(@file, "r")
	offset = 0

	while offset = self.extract_single(dir + "/part#{$part}", mmap, offset)
		yield dir + "/part#{$part}" if block_given?
		$part += 1
	end
    end
end

if __FILE__ == $0
    require 'getopts'

    $part = 0	#global counter
    
    if (!getopts('vh', 'o:') || ARGV.size != 1 || $OPT_h) then
        $stderr.print("Usage: #{$0} [-vh] -o dir file")
        exit 1
    end

    curpwd = Dir.pwd
    if curpwd.nil? then
	$stderr.print("Can't get pwd")
	exit 1
    end

    outdir = $OPT_o || curpwd
    mcfile = ARGV[0]
    ncodes = 0

    tmpdir = `/usr/bin/mktemp -d /tmp/devcpu.XXXX`.chomp!
    if tmpdir.empty? then
        $stderr.print("Can't create temporary directory")
        exit 1
    end

    print "Created temp directory #{tmpdir}\n" if $OPT_v

    AwardBiosReader.new(mcfile).extract(tmpdir) do |part|
	Dir.chdir(tmpdir)
        LhaLib::x(part) do |file|
	    Dir.chdir(curpwd)
	    filepath = tmpdir + "/" + file[:name]
	    AmdCpucodeReader.new(filepath).extract(outdir) do |code|
		print "Extracting...\n" if $OPT_v and ncodes.zero?
            	print "	amd:#{code}\n" if $OPT_v
		ncodes += 1
	    end
	    IntelCpucodeReader.new(filepath).extract(outdir) do |code|
		print "Extracting...\n" if $OPT_v and ncodes.zero?
            	print "	intel:#{code}\n" if $OPT_v
		ncodes += 1
	    end
	    File.unlink(filepath)
        end
	File.unlink(part)
    end

    Dir.rmdir(tmpdir)
    print "Temp directory deleted.\n" if $OPT_v
    print "#{ncodes} mcode files extracted.\n" if $OPT_v
end
